/**
    @file ft51_iomux.h

    @brief IOMUX library

    Maps signals to pads and configures their electrical characteristics.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_IOMUX_H__
#define __FT51_IOMUX_H__

// To be available to all includers of this file, the PAD, IN_SIGNAL and 
// OUT_SIGNAL typedefs precede any #includes.

/**
    @enum   IOMUX_PAD
    @brief  IOMUX pads available for mapping I/O signals.
            The pads are connected to physical pins on the device.  Not all
            pads are available on external pins on all package types.
            To work out which pin number is associated with each pad please
            refer to the datasheet.
**/
typedef enum
{
    /** Analog/Digital pads. **/
    AIO_0,
    AIO_1,
    AIO_2,
    AIO_3,
    AIO_4,
    AIO_5,
    AIO_6,
    AIO_7,
    AIO_8,
    AIO_9,
    AIO_10,
    AIO_11,
    AIO_12,
    AIO_13,
    AIO_14,
    AIO_15,
    /** Digital only pads. **/
    DIO_0,
    DIO_1,
    DIO_2,
    DIO_3,
    DIO_4,
    DIO_5,
    DIO_6,
    DIO_7,
    DIO_8,
    DIO_9,
    DIO_10,
    DIO_11,
    DIO_12,
    DIO_13,
    DIO_14,
    DIO_15,
} IOMUX_PAD;

/**
    @enum  IOMUX_OUT_SIGNAL
    @brief Internal output signals to map to output pads.
**/
typedef enum
{
    IOMUX_OUT_PORT0_0, // 0
    IOMUX_OUT_PORT0_1,
    IOMUX_OUT_PORT0_2,
    IOMUX_OUT_PORT0_3,
    IOMUX_OUT_PORT0_4,
    IOMUX_OUT_PORT0_5, // 5
    IOMUX_OUT_PORT0_6,
    IOMUX_OUT_PORT0_7,
    IOMUX_OUT_PORT1_0,
    IOMUX_OUT_PORT1_1,
    IOMUX_OUT_PORT1_2, // 10
    IOMUX_OUT_PORT1_3,
    IOMUX_OUT_PORT1_4,
    IOMUX_OUT_PORT1_5,
    IOMUX_OUT_PORT1_6,
    IOMUX_OUT_PORT1_7, // 15
    IOMUX_OUT_UART_8051_TXD0,
    IOMUX_OUT_UART_8051_RXD0O,
    IOMUX_OUT_UART_TXD,
    IOMUX_OUT_UART_RTS_N,
    IOMUX_OUT_UART_DTR_N, // 20
    IOMUX_OUT_UART_TX_ACTIVE,
    IOMUX_OUT_GL_N,
    IOMUX_OUT_SUSPEND,
    IOMUX_OUT_SUSPEND_OPEN_DRAIN,
    IOMUX_OUT_CLKOUT, // 25
    IOMUX_OUT_SPI_MASTER_MOSI,
    IOMUX_OUT_SPI_MASTER_MISO_LOOPBACK,
    IOMUX_OUT_SPI_MASTER_SCLK,
    IOMUX_OUT_SPI_MASTER_SS_N_3,
    IOMUX_OUT_SPI_MASTER_SS_N_2, // 30
    IOMUX_OUT_SPI_MASTER_SS_N_1,
    IOMUX_OUT_SPI_MASTER_SS_N_0,
    IOMUX_OUT_SPI_SLAVE_MISO,
    IOMUX_OUT_SPI_SLAVE_MOSI_OUT,
    IOMUX_OUT_I2C_SCL, // 35
    IOMUX_OUT_I2C_SDA,
    IOMUX_OUT_I2C_MASTER_SCLH,
    IOMUX_OUT_PWM_OUT_0,
    IOMUX_OUT_PWM_OUT_1,
    IOMUX_OUT_PWM_OUT_2, // 40
    IOMUX_OUT_PWM_OUT_3,
    IOMUX_OUT_PWM_OUT_4,
    IOMUX_OUT_PWM_OUT_5,
    IOMUX_OUT_PWM_OUT_6,
    IOMUX_OUT_PWM_OUT_7, // 45
    IOMUX_OUT_FIFO_245_DATA_READ_0,
    IOMUX_OUT_FIFO_245_DATA_READ_1,
    IOMUX_OUT_FIFO_245_DATA_READ_2,
    IOMUX_OUT_FIFO_245_DATA_READ_3,
    IOMUX_OUT_FIFO_245_DATA_READ_4, // 50
    IOMUX_OUT_FIFO_245_DATA_READ_5,
    IOMUX_OUT_FIFO_245_DATA_READ_6,
    IOMUX_OUT_FIFO_245_DATA_READ_7,
    IOMUX_OUT_FIFO_245_TXE_N,
    IOMUX_OUT_FIFO_245_RXF_N, // 55
    IOMUX_OUT_MISC_BCD, // 56
    IOMUX_OUT_MISC_DEBUGGER = 60,
    IOMUX_OUT_MISC_LOW,
    IOMUX_OUT_MISC_HIGH,
    IOMUX_OUT_MISC_TRISTATE,
    IOMUX_OUT_PORT2_0,
    IOMUX_OUT_PORT2_1, // 65
    IOMUX_OUT_PORT2_2,
    IOMUX_OUT_PORT2_3,
    IOMUX_OUT_PORT2_4,
    IOMUX_OUT_PORT2_5,
    IOMUX_OUT_PORT2_6, // 70
    IOMUX_OUT_PORT2_7,
    IOMUX_OUT_PORT3_0,
    IOMUX_OUT_PORT3_1,
    IOMUX_OUT_PORT3_2,
    IOMUX_OUT_PORT3_3, // 75
    IOMUX_OUT_PORT3_4,
    IOMUX_OUT_PORT3_5,
    IOMUX_OUT_PORT3_6,
    IOMUX_OUT_PORT3_7,
    IOMUX_OUT_HUB_P2_PWREN, // 80
    IOMUX_OUT_I2C_MASTER_SCL,  
    IOMUX_OUT_I2C_MASTER_SDA,
    IOMUX_OUT_NOT_USED = -1,
} IOMUX_OUT_SIGNAL;

/**
    @enum  IOMUX_IN_SIGNAL
    @brief Internal input signals to map from input pads.
**/
typedef enum
{   
    IOMUX_IN_PORT0_0, // 0
    IOMUX_IN_PORT0_1,
    IOMUX_IN_PORT0_2,
    IOMUX_IN_PORT0_3,
    IOMUX_IN_PORT0_4,
    IOMUX_IN_PORT0_5, // 5
    IOMUX_IN_PORT0_6,
    IOMUX_IN_PORT0_7,
    IOMUX_IN_PORT1_0,
    IOMUX_IN_PORT1_1,
    IOMUX_IN_PORT1_2, // 10
    IOMUX_IN_PORT1_3,
    IOMUX_IN_PORT1_4,
    IOMUX_IN_PORT1_5,
    IOMUX_IN_PORT1_6,
    IOMUX_IN_PORT1_7, // 15
    IOMUX_IN_UART_8051_RXD0I,
    IOMUX_IN_UART_RXD,
    IOMUX_IN_UART_CTS_N,
    IOMUX_IN_UART_DSR_N,
    IOMUX_IN_UART_RI, // 20
    IOMUX_IN_UART_DCD,
    IOMUX_IN_SPI_MASTER_MISO,
    IOMUX_IN_SPI_SLAVE_MOSI_IN,
    IOMUX_IN_SPI_SLAVE_SCLK,
    IOMUX_IN_SPI_SLAVE_SS_N, // 25
    IOMUX_IN_I2C_SCL,
    IOMUX_IN_I2C_SDA,
    IOMUX_IN_FIFO_245_DATA_WRITE_0,
    IOMUX_IN_FIFO_245_DATA_WRITE_1,
    IOMUX_IN_FIFO_245_DATA_WRITE_2, // 30
    IOMUX_IN_FIFO_245_DATA_WRITE_3,
    IOMUX_IN_FIFO_245_DATA_WRITE_4,
    IOMUX_IN_FIFO_245_DATA_WRITE_5,
    IOMUX_IN_FIFO_245_DATA_WRITE_6,
    IOMUX_IN_FIFO_245_DATA_WRITE_7, // 35
    IOMUX_IN_FIFO_245_WR_N,
    IOMUX_IN_FIFO_245_RD_N,
    IOMUX_IN_MISC_WAKEUP_PIN,
    IOMUX_IN_MISC_PWM_TRIGGER,
    IOMUX_IN_MISC_VBUS_DETECTED, // 40
    IOMUX_IN_PORT2_0,
    IOMUX_IN_PORT2_1,
    IOMUX_IN_PORT2_2,
    IOMUX_IN_PORT2_3,
    IOMUX_IN_PORT2_4, // 45
    IOMUX_IN_PORT2_5,
    IOMUX_IN_PORT2_6,
    IOMUX_IN_PORT2_7,
    IOMUX_IN_PORT3_0,
    IOMUX_IN_PORT3_1, // 50
    IOMUX_IN_PORT3_2,
    IOMUX_IN_PORT3_3,
    IOMUX_IN_PORT3_4,
    IOMUX_IN_PORT3_5,
    IOMUX_IN_PORT3_6, // 55
    IOMUX_IN_PORT3_7,
    IOMUX_IN_HUB_P1_OVER_CURRENT,
    IOMUX_IN_HUB_P2_OVER_CURRENT,
    IOMUX_IN_I2C_MASTER_SCL,  
    IOMUX_IN_I2C_MASTER_SDA, // 60
    IOMUX_IN_NOT_USED = -1,
} IOMUX_IN_SIGNAL;

//////////////////////////////////////////////////////////////////

#include <stdint.h>
#include "ft51_io_registers.h"
#include "registers/ft51_iomux_registers.h"

//////////////////////////////////////////////////////////////////
#define IOMUX_OUTPUT(pad, signal) \
    do { \
        IO_REG_GENERAL_WRITE(IOMUX_OUTPUT_PAD_SEL, pad); \
        IO_REG_GENERAL_WRITE(IOMUX_OUTPUT_SIG_SEL, signal); \
    } while (0)

//////////////////////////////////////////////////////////////////
#define IOMUX_INPUT(pad, signal) \
    do { \
        IO_REG_GENERAL_WRITE(IOMUX_INPUT_SIG_SEL, signal); \
        IO_REG_GENERAL_WRITE(IOMUX_INPUT_PAD_SEL, pad); \
    } while (0)

//////////////////////////////////////////////////////////////////
#define IOMUX_RESET \
    do { \
        IO_REG_GENERAL_WRITE(IOMUX_CONTROL, MASK_IOMUX_SOFT_RESET); \
    } while (0)

//////////////////////////////////////////////////////////////////
#define IOMUX_ENABLE \
    do { \
        IO_REG_GENERAL_WRITE(IOMUX_CONTROL, MASK_IOMUX_DEV_EN); \
    } while (0)

//////////////////////////////////////////////////////////////////

#define IOMUX_CONFIG_x {\
    /* PAD  */ /* IN SIGNAL, OUT_SIGNAL */ \
    /*AIO_0 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_1 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_2 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_3 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_6 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_7 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_8 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_9 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_10*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_11*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_12*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_13*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_14*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*AIO_15*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_0 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_1 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_2 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_3 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_6 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_7 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_8 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_9 */ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_10*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_11*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_12*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_13*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_14*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
    /*DIO_15*/ IOMUX_IN_NOT_USED, IOMUX_OUT_NOT_USED, \
}

#define IOMUX_CONFIG_3 {\
    /* PAD  */ /* IN SIGNAL, OUT_SIGNAL */ \
    /*AIO_0 */ IOMUX_IN_I2C_SDA, IOMUX_OUT_SIGNAL, \
    /*AIO_1 */ IOMUX_IN_I2C_SCL, IOMUX_OUT_SIGNAL, \
    /*AIO_2 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*AIO_3 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*AIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_SLAVE_MISO, \
    /*AIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_SLAVE_MOSI_OUT, \
    /*AIO_6 */ IOMUX_IN_SPI_SLAVE_MOSI_IN, IOMUX_OUT_SIGNAL, \
    /*AIO_7 */ IOMUX_IN_SPI_SLAVE_SCLK, IOMUX_OUT_SIGNAL, \
    /*AIO_8 */ IOMUX_IN_UART_DCD, IOMUX_OUT_SIGNAL, \
    /*AIO_9 */ IOMUX_IN_UART_CTS_N, IOMUX_OUT_SIGNAL, \
    /*AIO_10*/ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_TXD, \
    /*AIO_11*/ IOMUX_IN_UART_RXD, IOMUX_OUT_SIGNAL, \
    /*AIO_12*/ IOMUX_IN_UART_RI, IOMUX_OUT_SIGNAL, \
    /*AIO_13*/ IOMUX_IN_UART_DSR_N, IOMUX_OUT_SIGNAL, \
    /*AIO_14*/ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_TX_ACTIVE, \
    /*AIO_15*/ IOMUX_IN_SPI_SLAVE_SS_N, IOMUX_OUT_SIGNAL, \
    /*DIO_0 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_MOSI, \
    /*DIO_1 */ IOMUX_IN_SPI_MASTER_MISO, IOMUX_OUT_SIGNAL, \
    /*DIO_2 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_SCLK, \
    /*DIO_3 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_SS_N_0, \
    /*DIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_6 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_7 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_8 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_9 */ IOMUX_IN_I2C_MASTER_SDA, IOMUX_OUT_I2C_MASTER_SDA, \
    /*DIO_10*/ IOMUX_IN_I2C_MASTER_SCL, IOMUX_OUT_I2C_MASTER_SCL, \
    /*DIO_11*/ IOMUX_IN_NOT_USED, IOMUX_OUT_I2C_MASTER_SCLH, \
    /*DIO_12*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_13*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_14*/ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_DTR_N, \
    /*DIO_15*/ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_RTS_N, \
}

#define IOMUX_CONFIG_2 {\
    /* PAD  */ /* IN SIGNAL, OUT_SIGNAL */ \
    /*AIO_0 */ IOMUX_IN_NOT_USED, IOMUX_OUT_PWM_OUT_0, \
    /*AIO_1 */ IOMUX_IN_NOT_USED, IOMUX_OUT_PWM_OUT_1, \
    /*AIO_2 */ IOMUX_IN_NOT_USED, IOMUX_OUT_PWM_OUT_2, \
    /*AIO_3 */ IOMUX_IN_MISC_PWM_TRIGGER, IOMUX_OUT_SIGNAL, \
    /*AIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_SS_N_0, \
    /*AIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_SCLK, \
    /*AIO_6 */ IOMUX_IN_SPI_MASTER_MISO, IOMUX_OUT_SIGNAL, \
    /*AIO_7 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_MOSI, \
    /*AIO_8 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*AIO_9 */ IOMUX_IN_SPI_SLAVE_MOSI_IN, IOMUX_OUT_SIGNAL, \
    /*AIO_10*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_SLAVE_MOSI_OUT, \
    /*AIO_11*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_SLAVE_MISO, \
    /*AIO_12*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*AIO_13*/ IOMUX_IN_NOT_USED, IOMUX_OUT_SPI_MASTER_SS_N_1, \
    /*AIO_14*/ IOMUX_IN_I2C_SCL, IOMUX_OUT_SIGNAL, \
    /*AIO_15*/ IOMUX_IN_I2C_SDA, IOMUX_OUT_SIGNAL, \
    /*DIO_0 */ IOMUX_IN_NOT_USED, IOMUX_OUT_I2C_MASTER_SCLH, \
    /*DIO_1 */ IOMUX_IN_I2C_MASTER_SCL, IOMUX_OUT_I2C_MASTER_SCL, \
    /*DIO_2 */ IOMUX_IN_I2C_MASTER_SDA, IOMUX_OUT_I2C_MASTER_SDA, \
    /*DIO_3 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_4 */ IOMUX_IN_NOT_USED, IOMUX_OUT_SIGNAL, \
    /*DIO_5 */ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_DTR_N, \
    /*DIO_6 */ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_RTS_N, \
    /*DIO_7 */ IOMUX_IN_UART_DCD, IOMUX_OUT_SIGNAL, \
    /*DIO_8 */ IOMUX_IN_UART_RXD, IOMUX_OUT_SIGNAL, \
    /*DIO_9 */ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_TXD, \
    /*DIO_10*/ IOMUX_IN_UART_CTS_N, IOMUX_OUT_SIGNAL, \
    /*DIO_11*/ IOMUX_IN_UART_RI, IOMUX_OUT_SIGNAL, \
    /*DIO_12*/ IOMUX_IN_UART_DSR_N, IOMUX_OUT_SIGNAL, \
    /*DIO_13*/ IOMUX_IN_NOT_USED, IOMUX_OUT_UART_TX_ACTIVE, \
    /*DIO_14*/ IOMUX_IN_SPI_SLAVE_SS_N, IOMUX_OUT_SIGNAL, \
    /*DIO_15*/ IOMUX_IN_SPI_SLAVE_SCLK, IOMUX_OUT_SIGNAL, \
}

#define IOMUX_CONFIG_1 {\
    /* PAD  */ /* IN SIGNAL, OUT_SIGNAL */ \
    /*AIO_0 */ IOMUX_IN_PORT0_0, IOMUX_OUT_PORT0_0, \
    /*AIO_1 */ IOMUX_IN_PORT0_1, IOMUX_OUT_PORT0_1, \
    /*AIO_2 */ IOMUX_IN_PORT0_2, IOMUX_OUT_PORT0_2, \
    /*AIO_3 */ IOMUX_IN_PORT0_3, IOMUX_OUT_PORT0_3, \
    /*AIO_4 */ IOMUX_IN_PORT0_4, IOMUX_OUT_PORT0_4, \
    /*AIO_5 */ IOMUX_IN_PORT0_5, IOMUX_OUT_PORT0_5, \
    /*AIO_6 */ IOMUX_IN_PORT0_6, IOMUX_OUT_PORT0_6, \
    /*AIO_7 */ IOMUX_IN_PORT0_7, IOMUX_OUT_PORT0_7, \
    /*AIO_8 */ IOMUX_IN_PORT2_0, IOMUX_OUT_PORT2_0, \
    /*AIO_9 */ IOMUX_IN_PORT2_1, IOMUX_OUT_PORT2_1, \
    /*AIO_10*/ IOMUX_IN_PORT2_2, IOMUX_OUT_PORT2_2, \
    /*AIO_11*/ IOMUX_IN_PORT2_3, IOMUX_OUT_PORT2_3, \
    /*AIO_12*/ IOMUX_IN_PORT2_4, IOMUX_OUT_PORT2_4, \
    /*AIO_13*/ IOMUX_IN_PORT2_5, IOMUX_OUT_PORT2_5, \
    /*AIO_14*/ IOMUX_IN_PORT2_6, IOMUX_OUT_PORT2_6, \
    /*AIO_15*/ IOMUX_IN_PORT2_7, IOMUX_OUT_PORT2_7, \
    /*DIO_0 */ IOMUX_IN_PORT1_0, IOMUX_OUT_PORT1_0, \
    /*DIO_1 */ IOMUX_IN_PORT1_1, IOMUX_OUT_PORT1_1, \
    /*DIO_2 */ IOMUX_IN_PORT1_2, IOMUX_OUT_PORT1_2, \
    /*DIO_3 */ IOMUX_IN_PORT1_3, IOMUX_OUT_PORT1_3, \
    /*DIO_4 */ IOMUX_IN_PORT1_4, IOMUX_OUT_PORT1_4, \
    /*DIO_5 */ IOMUX_IN_PORT1_5, IOMUX_OUT_PORT1_5, \
    /*DIO_6 */ IOMUX_IN_PORT1_6, IOMUX_OUT_PORT1_6, \
    /*DIO_7 */ IOMUX_IN_PORT1_7, IOMUX_OUT_PORT1_7, \
    /*DIO_8 */ IOMUX_IN_PORT2_0, IOMUX_OUT_PORT2_0, \
    /*DIO_9 */ IOMUX_IN_PORT2_1, IOMUX_OUT_PORT2_1, \
    /*DIO_10*/ IOMUX_IN_PORT2_2, IOMUX_OUT_PORT2_2, \
    /*DIO_11*/ IOMUX_IN_PORT2_3, IOMUX_OUT_PORT2_3, \
    /*DIO_12*/ IOMUX_IN_PORT2_4, IOMUX_OUT_PORT2_4, \
    /*DIO_13*/ IOMUX_IN_PORT2_5, IOMUX_OUT_PORT2_5, \
    /*DIO_14*/ IOMUX_IN_PORT2_6, IOMUX_OUT_PORT2_6, \
    /*DIO_15*/ IOMUX_IN_PORT2_7, IOMUX_OUT_PORT2_7, \
}

#define IOMUX_SETUP_PAD(PAD, SIGNAL_IN, SIGNAL_OUT) \
    do { \
        IO_REG_GENERAL_WRITE(IOMUX_OUTPUT_PAD_SEL, PAD); \
        IO_REG_GENERAL_WRITE(IOMUX_OUTPUT_SIG_SEL, SIGNAL_OUT); \
        IO_REG_GENERAL_WRITE(IOMUX_INPUT_SIG_SEL, SIGNAL_IN); \
        IO_REG_GENERAL_WRITE(IOMUX_INPUT_PAD_SEL, PAD); \
    } while (0)


// call with the following format:
    // IOMUX_CONFIG(IOMUX_CONFIG_1);
#define IOMUX_CONFIG(IOMUX_CONFIG_ARRAY) \
    do { \
        char reg[] = IOMUX_CONFIG_ARRAY; \
        uint8_t i; \
        for (i = 0; i<sizeof(reg); i+=2) \
        { \
            IOMUX_SETUP_PAD(i>>1, reg[i], reg[i+1]); \
        } \
    } while(0)

#define IOMUX_PAD_TO_PIN { \
            /*    48, 44, 32, 28 */\
    /*AIO_0*/    40, 37, 0,  0,\
    /*AIO_1*/    39, 36, 0,  0,\
    /*AIO_2*/    38, 35, 0,  0,\
    /*AIO_3*/    37, 34, 0,  0,\
    /*AIO_4*/    36, 33, 25, 26,\
    /*AIO_5*/    35, 32, 24, 25,\
    /*AIO_6*/    34, 31, 23, 24,\
    /*AIO_7*/    33, 30, 22, 23,\
    /*AIO_8*/    20, 18, 0, 0,\
    /*AIO_9*/    21, 19, 0, 0,\
    /*AIO_10*/    22, 20, 16, 17,\
    /*AIO_11*/    23, 21, 17, 18,\
    /*AIO_12*/    24, 22, 0, 0,\
    /*AIO_13*/    25, 23, 0, 0,\
    /*AIO_14*/    26, 24, 18, 19,\
    /*AIO_15*/    27, 25, 19, 20,\
    /*DIO_0*/    45, 40, 28, 1,\
    /*DIO_1*/    46, 41, 29, 2,\
    /*DIO_2*/    47, 42, 30, 3,\
    /*DIO_3*/    48, 43, 31, 4,\
    /*DIO_4*/    1,  44, 32, 0,\
    /*DIO_5*/    2,  1,  1,  0,\
    /*DIO_6*/    3,  2,  2,  0,\
    /*DIO_7*/    4,  3,  3,  0,\
    /*DIO_8*/    6,  5,  5,  6,\
    /*DIO_9*/    9,  7,  6,  8,\
    /*DIO_10*/    10, 8,  7,  9,\
    /*DIO_11*/    11, 9,  8,  10,\
    /*DIO_12*/    12, 10, 9,  11,\
    /*DIO_13*/    13, 11, 10, 12,\
    /*DIO_14*/    14, 12, 11, 13,\
    /*DIO_15*/    15, 13, 12, 14,\
}

#define IOMUX_PIN_TO_PAD { \
            /*    48,     44,     32,     28 */\
        /* 1*/    DIO_4,    DIO_5,     DIO_5,     DIO_0,\
        /* 2*/    DIO_5,     DIO_6,    DIO_6,     DIO_1,\
        /* 3*/    DIO_6,     DIO_7,    DIO_7,     DIO_2,\
        /* 4*/    DIO_7,     DIO_9,    DIO_10,    DIO_3,\
        /* 5*/    -1,     DIO_8,    DIO_8,     -1,\
        /* 6*/    DIO_8,    -1,     DIO_9,     DIO_8,\
        /* 7*/    -1,     DIO_9,     DIO_10,    -1,\
        /* 8*/    -1,     DIO_10,    DIO_11,    DIO_9,\
        /* 9*/    DIO_9,     DIO_11,    DIO_12,    DIO_10,\
        /*10*/    DIO_10,    DIO_12,    DIO_13,    DIO_11,\
        /*11*/    DIO_11,    DIO_13,    DIO_14,    DIO_12,\
        /*12*/    DIO_12,    DIO_14,    DIO_15, DIO_13,\
        /*13*/    DIO_13,    DIO_15,    -1,        DIO_14,\
        /*14*/    DIO_14,    -1,        -1,     DIO_15,\
        /*15*/    DIO_15, -1,        -1,        -1,\
        /*16*/    -1,     -1,     AIO_10, -1,\
        /*17*/    -1,     -1,     AIO_11,    AIO_10,\
        /*18*/    -1,     AIO_8,  AIO_14,    AIO_11,\
        /*19*/    -1,     AIO_9,     AIO_15,    AIO_14,\
        /*20*/    AIO_8,     AIO_10,    -1,     AIO_15,\
        /*21*/    AIO_9,     AIO_11,    -1,     -1,\
        /*22*/    AIO_10, AIO_12,    AIO_7,     -1,\
        /*23*/    AIO_11,    AIO_13,    AIO_6,     AIO_7,\
        /*24*/    AIO_12,    AIO_14,    AIO_5,     AIO_6,\
        /*25*/    AIO_13,    AIO_15,    AIO_4,     AIO_5,\
        /*26*/    AIO_14,    -1,     -1,     AIO_4,\
        /*27*/    AIO_15,    -1,     -1,     -1,\
        /*28*/    -1,     -1,     DIO_0,     -1,\
        /*29*/    -1,     -1,     DIO_1,     -1,\
        /*30*/    -1,     AIO_7,     DIO_2,     -1,\
        /*31*/    -1,     AIO_6,     DIO_3,     -1,\
        /*32*/    -1,     AIO_5,     DIO_4,     -1,\
        /*33*/    AIO_7,     AIO_4,     -1,     -1,\
        /*34*/    AIO_6,     AIO_3,     -1,     -1,\
        /*35*/    AIO_5,     AIO_2,     -1,     -1,\
        /*36*/    AIO_4,     AIO_1,     -1,     -1,\
        /*37*/    AIO_3,     AIO_0,     -1,     -1,\
        /*38*/    AIO_2,     -1,     -1,     -1,\
        /*39*/    AIO_1,     -1,     -1,     -1,\
        /*40*/    AIO_0,     DIO_0,     -1,     -1,\
        /*41*/    -1,     DIO_1,     -1,     -1,\
        /*42*/    -1,     DIO_2,     -1,     -1,\
        /*43*/    -1,     DIO_3,     -1,     -1,\
        /*44*/    -1,     DIO_4,     -1,     -1,\
        /*45*/    DIO_0,     -1,     -1,     -1,\
        /*46*/    DIO_1,     -1,     -1,     -1,\
        /*47*/    DIO_2,     -1,     -1,     -1,\
        /*48*/    DIO_3,     -1,     -1,     -1,\
    }

/**
    @enum  IOMUX_PAD_DRIVE_CURRENT
    @brief IOMUX pad driver current definitions for output pads.
**/
typedef enum
{
    IOMUX_PAD_DRIVE_CURRENT_4MA,
    IOMUX_PAD_DRIVE_CURRENT_8MA,
    IOMUX_PAD_DRIVE_CURRENT_12MA,
    IOMUX_PAD_DRIVE_CURRENT_16MA,
} IOMUX_PAD_DRIVE_CURRENT;

/**
    @enum  IOMUX_PAD_TRIGGER
    @brief IOMUX pad trigger definitions for input pads.
**/
typedef enum
{
    IOMUX_PAD_TRIGGER_NORMAL,
    IOMUX_PAD_TRIGGER_SCHMITT,
} IOMUX_PAD_TRIGGER;


/**
    @enum  IOMUX_PAD_SLEW_RATE
    @brief IOMUX pad slew rate definitions for output pads.
**/
typedef enum
{
    IOMUX_PAD_SLEW_RATE_FAST,
    IOMUX_PAD_SLEW_RATE_SLOW,
} IOMUX_PAD_SLEW_RATE;

/**
    @enum  IOMUX_PAD_PULL
    @brief IOMUX pad pull up/down definitions for input pads.
**/
typedef enum
{
    /** No pull up or pull down. **/
    IOMUX_PAD_PULL_NONE,
    /** Pull up. **/
    IOMUX_PAD_PULL_UP,
    /** Pull down. **/
    IOMUX_PAD_PULL_DOWN,
    /** Pull down and up. **/
    IOMUX_PAD_PULL_KEEPER,
} IOMUX_PAD_PULL;

/**
    @brief      Initialise IOMUX hardware.

    This function must be called before any call to the other IOMUX 
    functions.  Internally calls IOMUX_RESET().

    @return     void
**/
void IOMUX_initialise();

/**
   @brief          IOMUX_get_pad_config

   Obtains the characteristics of the pad.

   @param[in]      pad         The pad number to test.
   @param[out]     drive       The drive current configured.
   @param[out]     trigger     The trigger type configured.
   @param[out]     slew        The slew rate configured.
   @param[out]     pull        The pull up mode configured.
   @return         FT51_STATUS
**/
void IOMUX_get_pad_config(const IOMUX_PAD pad,
                                IOMUX_PAD_DRIVE_CURRENT *drive,
                                IOMUX_PAD_TRIGGER *trigger,
                                IOMUX_PAD_SLEW_RATE *slew,
                                IOMUX_PAD_PULL *pull);

/**
   @brief          IOMUX_set_pad_config

   Changes the characteristics of the pad.

   @param[in]      pad         The pad number to configure.
   @param[in]      drive       The drive current to configure.
   @param[in]      trigger     The trigger type to configure.
   @param[in]      slew        The slew rate to configure.
   @param[in]      pull        The pull up mode to configure.
   @return         void
**/
void IOMUX_set_pad_config(const IOMUX_PAD pad,
                                IOMUX_PAD_DRIVE_CURRENT drive,
                                IOMUX_PAD_TRIGGER trigger,
                                IOMUX_PAD_SLEW_RATE slew,
                                IOMUX_PAD_PULL pull);

#endif // __FT51_IOMUX_H__