/**
    @file ft51_interrupts.c

    @brief Interrupt Library API interrupt handler functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdlib.h>
#include <stdint.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "registers/ft51_registers.h"
#include "ft51_io_registers.h"
#include "ft51_interrupts.h"

// To avoid bloating the binary, selectively enable interrupts by building
// with FT51_INTERRUPTS set to a non-zero value.  See ft51_interrupts.h.
// By default, all interrupts are enabled.

// Classic 8051 Interrupt-Enable register bits.
enum IE_BITS
{
    IE_GLOBAL     = 1 << 7,
    // bit 6 not defined
    IE_TIMER_2    = 1 << 5,
    IE_SERIAL     = 1 << 4,
    IE_TIMER_1    = 1 << 3,
    IE_EXTERNAL_1 = 1 << 2,
    IE_TIMER_0    = 1 << 1,
    IE_EXTERNAL_0 = 1
};

// Extended 8051 Interrupt-Enable register bits.
enum EIE_BITS
{
    EIE_I2C_SLAVE  = 1 << 7,
    EIE_I2C_MASTER = 1 << 6
};

#if (FT51_INTERRUPTS & FT51_INT_DMA) == FT51_INT_DMA
static interrupts_handler   interrupts_DMA_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_DMA)
#if (FT51_INTERRUPTS & FT51_INT_TIMER0) == FT51_INT_TIMER0
static interrupts_handler   interrupts_timer0_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER0)
#if (FT51_INTERRUPTS & FT51_INT_TIMER1) == FT51_INT_TIMER1
static interrupts_handler   interrupts_timer1_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER1)
#if (FT51_INTERRUPTS & FT51_INT_TIMER2) == FT51_INT_TIMER2
static interrupts_handler   interrupts_timer2_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER2)
#if (FT51_INTERRUPTS & FT51_INT_SERIAL) == FT51_INT_SERIAL
static interrupts_handler   interrupts_serial_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_SERIAL)
#if (FT51_INTERRUPTS & FT51_INT_PERIPHERALS) == FT51_INT_PERIPHERALS
/* FT122, IO_CELL_CONTROLLER, Enhanced UART, SPI_MASTER,
   SPI_SLAVE, PWM, TIMER, FIFO_245
 */
static interrupts_handler   interrupts_int1_handlers[8];
// Variables to minimise scan time in INT1_ISR
static uint8_t first_registered = BIT_FIFO_245_IRQ;
static uint8_t last_registered = BIT_FT122_IRQ;
#endif // (FT51_INTERRUPTS & FT51_INT_PERIPHERALS)
#if (FT51_INTERRUPTS & FT51_INT_I2CM) == FT51_INT_I2CM
static interrupts_handler   interrupts_I2C_master_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_I2CM)
#if (FT51_INTERRUPTS & FT51_INT_I2CS) == FT51_INT_I2CS
static interrupts_handler   interrupts_I2C_slave_handler;
#endif // (FT51_INTERRUPTS & FT51_INT_I2CS)

void interrupts_register(interrupts_handler    isr, 
                         interrupts_peripheral peripheral)
{
    uint8_t value;

    switch (peripheral)
    {
#if (FT51_INTERRUPTS & FT51_INT_DMA) == FT51_INT_DMA
        case interrupts_DMA:
            interrupts_DMA_handler = isr;
            // 8051 core sees DMA as 'External 0' source of interrupts
            IE |= (IE_GLOBAL | IE_EXTERNAL_0);
            // Enable interrupts on all 4 DMA engines
            IO_REG_GENERAL_READ(PERIPHERAL_IEN0, value);
            value |= MASK_DMA0_IRQ_IEN |
                                    MASK_DMA1_IRQ_IEN | 
                                    MASK_DMA2_IRQ_IEN | 
                                    MASK_DMA3_IRQ_IEN;
            IO_REG_GENERAL_WRITE(PERIPHERAL_IEN0, value);
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_DMA)

#if (FT51_INTERRUPTS & FT51_INT_TIMER0) == FT51_INT_TIMER0
        case interrupts_timer0:
            interrupts_timer0_handler = isr;
            // Enable interrupts (bit 7: global, bit 1: Timer0 specifically)
            IE |= (IE_GLOBAL | IE_TIMER_0);
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER0)

#if (FT51_INTERRUPTS & FT51_INT_TIMER1) == FT51_INT_TIMER1
        case interrupts_timer1:
            interrupts_timer1_handler = isr;
            // Enable interrupts (bit 7: global, bit 3: Timer1 specifically)
            IE |= (IE_GLOBAL | IE_TIMER_1);
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER1)

#if (FT51_INTERRUPTS & FT51_INT_TIMER2) == FT51_INT_TIMER2
        case interrupts_timer2:
            interrupts_timer2_handler = isr;
            // Enable interrupts (global, and Timer2 specifically)
            IE |= (IE_GLOBAL | IE_TIMER_2);
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER1)

#if (FT51_INTERRUPTS & FT51_INT_SERIAL) == FT51_INT_SERIAL
        case interrupts_serial:
            interrupts_serial_handler = isr;
            // Enable interrupts (global, and serial port specifically)
            IE |= (IE_GLOBAL | IE_SERIAL);
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_SERIAL)

#if (FT51_INTERRUPTS & FT51_INT_PERIPHERALS) == FT51_INT_PERIPHERALS
        case interrupts_FT122:
        case interrupts_IO_cell:
        case interrupts_UART:
        case interrupts_SPI_master:
        case interrupts_SPI_slave:
        case interrupts_PWM:
        case interrupts_timer:
        case interrupts_parallel245:
        {
            uint8_t offset = peripheral - interrupts_FT122;

            interrupts_int1_handlers[offset] = isr;

            // Allow INT1_ISR to skip some unregistered peripherals.
            if (offset < first_registered)
                first_registered = offset;
            if (offset > last_registered)
                last_registered = offset;

                // 8051 core sees these as 'External 1' source of interrupts
            IE |= (IE_GLOBAL | IE_EXTERNAL_1);
            // Enable interrupts for this specific peripheral.
            IO_REG_GENERAL_READ(PERIPHERAL_IEN1, value);
            value |= 1 << offset;
            IO_REG_GENERAL_WRITE(PERIPHERAL_IEN1, value);
            break;
        }
#endif // (FT51_INTERRUPTS & FT51_INT_PERIPHERALS)

#if (FT51_INTERRUPTS & FT51_INT_I2CM) == FT51_INT_I2CM
        case interrupts_I2C_master:
            interrupts_I2C_master_handler = isr;
            // Enable 8051 global interrupts - is this necessary?
            IE |= IE_GLOBAL;
            EIE |= EIE_I2C_MASTER;
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_I2CM)

#if (FT51_INTERRUPTS & FT51_INT_I2CS) == FT51_INT_I2CS
        case interrupts_I2C_slave:
            interrupts_I2C_slave_handler = isr;
            // Enable 8051 global interrupts - is this necessary?
            IE |= IE_GLOBAL;
            EIE |= EIE_I2C_SLAVE;
            break;
#endif // (FT51_INTERRUPTS & FT51_INT_I2CS)            
    }
}
        
        
#if (FT51_INTERRUPTS & FT51_INT_DMA) == FT51_INT_DMA
void INT0_ISR(void) __interrupt (0) 
{
    uint8_t              events = 0;

    IO_REG_INTERRUPTS_READ(PERIPHERAL_INT0, events);

    events &= (MASK_DMA0_IRQ | MASK_DMA1_IRQ | MASK_DMA2_IRQ | MASK_DMA3_IRQ);
        
    if (events && interrupts_DMA_handler != NULL)
    {
        interrupts_DMA_handler(events);
    }
    
    // Clear the event flags
    IO_REG_INTERRUPTS_WRITE(PERIPHERAL_INT0, events);
}
#endif // (FT51_INTERRUPTS & FT51_INT_DMA)


#if (FT51_INTERRUPTS & FT51_INT_TIMER0) == FT51_INT_TIMER0
void Timer0_ISR(void) __interrupt (1) 
{
    uint8_t             info = 0;

    if (interrupts_timer0_handler != NULL)
    {
        interrupts_timer0_handler(info);
    }
    
    // No need to clear the interrupt flag (TCON bit 5): the 8051
    // ISR mechanism does this for us automatically.
}
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER0)


#if (FT51_INTERRUPTS & FT51_INT_PERIPHERALS) == FT51_INT_PERIPHERALS
/*
    Each of the 8 bits of INT1 indicates an interrupt from one of the 
    following peripherals.
    FT120, IO_CELL_CONTROLLER, UART, SPI_MASTER
    SPI_SLAVE, PWM, TIMER, FIFO_245
*/
void INT1_ISR(void) __interrupt (2) 
{
    interrupts_handler   int1_handler = NULL;
    uint8_t              events = 0;
    uint8_t              peripheral_specific = 0;
    uint8_t              bit;

    // Discover which peripherals have interrupted
    IO_REG_INTERRUPTS_READ(PERIPHERAL_INT1, events);

    // For each peripheral: if it interrupted, invoke its handler (if registered).
    for (bit = first_registered; bit <= last_registered; bit++)
    {
        uint8_t mask = (1 << bit);

        if ((events & mask) == 0)
            continue; // This peripheral did not interrupt.  Skip to next.
            
        int1_handler = interrupts_int1_handlers[bit];
        
        if (int1_handler != NULL)
        {
            int1_handler(peripheral_specific);
        }
    }

    IO_REG_INTERRUPTS_WRITE(PERIPHERAL_INT1, events); // Clear relevant interrupt flags
}
#endif // (FT51_INTERRUPTS & FT51_INT_PERIPHERALS)


#if (FT51_INTERRUPTS & FT51_INT_TIMER1) == FT51_INT_TIMER1
void Timer1_ISR(void) __interrupt (3) 
{
    uint8_t             info = 0;

    if (interrupts_timer1_handler != NULL)
    {
        interrupts_timer1_handler(info);
    }

    // No need to clear the interrupt flag (TCON bit 7): the 8051
    // ISR mechanism does this for us automatically.
}
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER1)


#if (FT51_INTERRUPTS & FT51_INT_SERIAL) == FT51_INT_SERIAL
void Serial_ISR(void) __interrupt (4) 
{
    uint8_t             info = 0;

    if (interrupts_serial_handler != NULL)
    {
        interrupts_serial_handler(info);
    }
}
#endif // (FT51_INTERRUPTS & FT51_INT_SERIAL)


#if (FT51_INTERRUPTS & FT51_INT_TIMER2) == FT51_INT_TIMER2
void Timer2_ISR(void) __interrupt (5)
{
    uint8_t             info = 0;

    if (interrupts_timer2_handler != NULL)
    {
        interrupts_timer2_handler(info);
    }

    // Must explicitly clear timer 2 interrupt flag.
    T2IF &= 0x7E;
}
#endif // (FT51_INTERRUPTS & FT51_INT_TIMER2)


#if (FT51_INTERRUPTS & FT51_INT_I2CM) == FT51_INT_I2CM
void I2C_MASTER_ISR(void) __interrupt (13) 
{
    uint8_t             info = 0;

    if (interrupts_I2C_master_handler != NULL)
    {
        interrupts_I2C_master_handler(info);
    }

    //Clear interrupt? EIF = MASK_I2C_MASTER_INT; 
}
#endif // (FT51_INTERRUPTS & FT51_INT_I2CM)


#if (FT51_INTERRUPTS & FT51_INT_I2CS) == FT51_INT_I2CS
void I2C_SLAVE_ISR(void) __interrupt (14) 
{
    uint8_t             info = 0;

    if (interrupts_I2C_slave_handler != NULL)
    {
        interrupts_I2C_slave_handler(info);
    }

    // Clear interrupt? EIF = MASK_I2C_SLAVE_INT;
}
#endif // (FT51_INTERRUPTS & FT51_INT_I2CS)